#!/usr/bin/perl

##############################################################################
#  Copyright (C) 2007 Paulo Cesar Pereira de Andrade.  All Rights Reserved.
#
#  This is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This software is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  Authors:
#	Paulo Cesar Pereira de Andrade <pcpa@mandriva.com.br>
##############################################################################

use Cwd;
use File::Basename;
use Getopt::Std;

use strict;


##############################################################################
# Check command line options
my %options;
my $command = "";


##############################################################################
# This hash table is used to allow creating aliases to commads
my %commands =
(
 "up"		=> "up",
 "update"	=> "up",
 "dist"		=> "dist",
 "checkdeps"	=> "checkdeps",
 "builddeps"	=> "builddeps",
 "build"	=> "build",
 );


##############################################################################
# Check -<letter> options and script command
Getopt::Std::getopts("vh?", \%options);
foreach (@ARGV) {
    if ($commands{$_}) {
	$command .= $commands{$_};
    }
    else {
	print("* Unknown command $_\n");
    }
}
if (not $commands{$command} or $options{'h'} or $options{'?'}) {
    print("Usage:\n",
	  File::Basename::basename($0), " [options ...] command\n",
	  "Options:\n",
	  " -v      Verbose mode.\n",
	  " -h, -?  Prints this message and quits.\n",
	  "Commands:",
	  " update    Updates source checkouts.\n",
	  " dist      Create tarball and patches.\n",
	  " checkdeps Create dependencies.\n",
	  " builddeps Build dependencies.\n",
	  " build     Rebuild package (binaries only, just ``make'' on BUILD/dir).\n",
	  );
    exit(0);
}

# For the moment, check it only once. Required for some fallback default
# value options.
die ("\$ENV{HOME} not set. Cannot continue.") unless ($ENV{HOME} ne "");


##############################################################################
# User configuration
sub git_config {
    my ($option) = @_;

    $option = `git-config --global --get $option`;
    $option =~ s/^\s+//;
    $option =~ s/\s+$//;

    return $option;
};

my $git_protocol = git_config("git.protocol");
$git_protocol = "git" unless ($git_protocol ne "");

my $git_repository = git_config("git.repository");
$git_repository = "git.mandriva.com/users/pcpa" unless ($git_repository ne "");

my $git_user_name = git_config("user.name");
die("* Please run:\n",
    "\tgit-config --global user.name = \"Your user name\"\n",
    "before proceeding.") unless ($git_user_name ne "");

my $git_user_email = git_config("user.email");
die("* Please run:\n",
    "\tgit-config --global user.email you\@yourdomain.example.com\n",
    "before proceeding.") unless ($git_user_email ne "");

# Example: /home/paulo/git.mandriva.com
my $x_gitdir = git_config("x.gitdir");
if ($x_gitdir eq "") {
    if ($options{'v'}) {
	print("* Please consider running:\n",
	      "\tgit-config --global x.gitdir /path/to/git/checkout/dir\n");
    }
    $x_gitdir = $ENV{HOME} . "/" . "mandriva/git";
}

# Example: /home/paulo/mandriva/svn
my $x_rpmdir = git_config("x.rpmdir");
if ($x_rpmdir eq "") {
    if ($options{'v'}) {
	print("* Please consider running:\n",
	      "\tgit-config --global x.rpmdir /path/to/rpm/checkout/dir\n");
    }
    $x_rpmdir = $ENV{HOME} . "/" . "mandriva/rpm";
}

# Default internal Mandriva value should be just:
# repsys co %n
# and it should be already properly set to use the default repsys ``protocol''
# of svn+ssh
my $rpm_checkout = git_config("rpm.checkout");
if ($rpm_checkout eq "") {
    if ($options{'v'}) {
	print("* Please consider running:\n",
	      "\tgit-config --global rpm.checkout \"command-line to-checkout where \%n means-package-name\"\n");
    }
    $rpm_checkout = "svn co http://svn.mandriva.com/svn/packages/cooker/\%n/current \%n";
}

my $rpm_update = git_config("rpm.update");
if ($rpm_update eq "") {
    if ($options{'v'}) {
	print("* Please consider running:\n",
	      "\tgit-config --global rpm.update \"command-line to-ensure rpm-checkout-is-up-toodate\"\n");
    }
    $rpm_update = "svn update";
}

if ($options{'v'}) {
    print("Using options:\n",
	  "  git_protocol   = $git_protocol\n",
	  "  git_repository = $git_repository\n",
	  "  git_user_name  = $git_user_name\n",
	  "  git_user_email = $git_user_email\n",
	  "  x_gitdir       = $x_gitdir\n",
	  "  x_rpmdir       = $x_rpmdir\n",
	  "  rpm_checkout   = $rpm_checkout\n",
	  "  rpm_update     = $rpm_update\n",
	  "\n");
}
die("$x_gitdir does not exist.") unless (-d $x_gitdir);
die("$x_rpmdir does not exist.") unless (-d $x_rpmdir);

##############################################################################
# System options that should be the same for all packagers, i.e. set only
# in this script.
my $git_upstream = "git://anongit.freedesktop.org/git";
my $compress     = "bzip2";
my $compress_ext = ".bz2";
my $x_depsdir    = "/usr/share/X11/mandriva";

if ($options{'v'}) {
    print("Using system options:\n",
	  "  upstream_git   = $git_upstream\n",
	  "  compress       = $compress\n",
	  "  compress_ext   = $compress_ext\n",
	  "\n");
}



##############################################################################
# List of modules. Note that it is stored as a list also to allow proper
# specification of order of packages that must be updated first.
# Fields are:
#	package		-> Name of mandriva rpm package
#	packname	-> Name of the package stored in rpm, used normally
#			   for libraries. -devel may or may not have the
#			   the format $packname-devel and this should
#			   be fixed, analyzed as the reasoning can be
#			   clear, i.e. binaries may need to coexist, but
#			   header files usually don't have versioning.
#	path		-> Pathname in git repository
#	name		-> Name of module (also directory to append to path)
#	version		-> Module version
#	prefix		-> Tarball name prefix. This should be reworked
#			   but for this first work, use it as a hack for
#			   tarballs that don't follow the pattern used by
#			   other packages. If not specified the default
#			   value of $name-$version is used.
#	tag		-> Specific upstream version being shiped
#			   tag fields defaults to $name-$version if not
#			   specified, but for clarity, maybe it is better
#			   to always specifiy it.
#	skip		-> Don't use this module as argument for script commands
#			   This should be temporary.
#	other values may be added, usually mainly for documentation, i.e.
#	things like 'TODO', 'BROKEN', etc.
#
# Assumptions:
#	branch mandriva+gpl exists and is up to date, i.e. if a new upstream
#	release is made, branch is merged with the proper bits
my @modules =
(
 {
     'package'	=> "x11-util-macros",
     'path'	=> "xorg/util",
     'name'	=> "macros",
     'version'	=> "1.1.5",
     # Needed to follow upstream convention that doesn't match other packages
     'prefix'	=> "util-macros-1.1.5",
     # Need tag as it doesn't match standard format and also, when set, may
     # also have a @mandriva extension
     'tag'	=> "util-macros-1.1.5",
 },
 #############################################################################
 # Protocol headers also have special handling
 #
 # FIXME: Currently there is the x11-proto-devel package that
 # has all <name>proto modules, but it should be converted to
 # a metapackage that requires new packages, ATM just suggested here
 {
     'package'	=> "x11-proto-applewm",
     'name'	=> "applewmproto",
     'path'	=> "xorg/proto",
     'version'	=> "1.0.3",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-fonts",
     'name'	=> "fontsproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-record",
     'name'	=> "recordproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.13.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xext",
     'name'	=> "xextproto",
     'path'	=>"xorg/proto",
     'version'	=> "7.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-bigreqs",
     'name'	=> "bigreqsproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-gl",
     'name'	=> "glproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.4.9",
     'tag'	=> "glproto-1.4.9",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-render",
     'name'	=> "renderproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.9.3",
     'tag'	=> "renderproto-0.9.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86bigfont",
     'name'	=> "xf86bigfontproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-calibrate",
     'name'	=> "calibrateproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.1.0",
     # no tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "x11-proto-input",
     'name'	=> "inputproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.4.2.1",
     'tag'	=> "inputproto-1.4.2.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-resource",
     'name'	=> "resourceproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86dga",
     'name'	=> "xf86dgaproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.3",
     'tag'	=> "xf86dgaproto-2.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-composite",
     'name'	=> "compositeproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.4",
     'tag'	=> "compositeproto-0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-kb",
     'name'	=> "kbproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.3",
     'tag'	=> "kbproto-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-scrnsaver",
     'name'	=> "scrnsaverproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.0",
     'tag'	=> "scrnsaver-1_1_0",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86dri",
     'name'	=> "xf86driproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.3",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-damage",
     'name'	=> "damageproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.0",
     'tag'	=> "damageproto-1.1.0",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-lg3d",
     'name'	=> "lg3dproto",
     'path'	=>"xorg/proto",
     'version'	=> "5.0",
      # no tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 # Probably should still ship to allow recompiling programs that
 # may access remote servers
 {
     'package'	=> "x11-proto-trap",
     'name'	=> "trapproto",
     'path'	=>"xorg/proto",
     'version'	=> "3.4.3",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86misc",
     'name'	=> "xf86miscproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.9.2",
      # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-dmx",
     'name'	=> "dmxproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     # no tag available
     'skip'	=> "ignore",
 },
 # No files in panoramixproto updated upstream?
 # Maybe renamed/moved to xineramaproto...
 {
     'package'	=> "x11-proto-video",
     'name'	=> "videoproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86rush",
     'name'	=> "xf86rushproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-evie",
     'name'	=> "evieproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-pm",
     'name'	=> "pmproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-windowswm",
     'name'	=> "windowswmproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.3",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xf86vidmode",
     'name'	=> "xf86vidmodeproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-fixes",
     'name'	=> "fixesproto",
     'path'	=>"xorg/proto",
     'version'	=> "4.0",
     # no available tag (existing tags way too old)
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-print",
     'name'	=> "printproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.3",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-x11",
     'name'	=> "x11proto",
     'path'	=>"xorg/proto",
     'version'	=> "7.0.11",
     'tag'	=> "x11proto-7.0.11",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-proto-xinerama",
     'name'	=> "xineramaproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     # no tag available
     'skip'	=> "ignore",
 },

 ##### Add here, or as another entry xcb-proto and vncproto

 #############################################################################
 # Libs don't have an specific order (just protos) but build in the order
 # of header dependencies
 {
     'package'	=> "libxau",
     'packname'	=> "libxau6",
     'path'	=> "xorg/lib",
     'name'	=> "libXau",
     'version'	=> "1.0.3",
     'tag'	=> "libXau-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libfs",
     'packname'	=> "libfs6",
     'path'	=> "xorg/lib",
     'name'	=> "libFS",
     'version'	=> "1.0.0",
     ## no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libice",
     'packname'	=> "libice6",
     'path'	=> "xorg/lib",
     'name'	=> "libICE",
     'version'	=> "1.0.4",
     'tag'	=> "libICE-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxdmcp",
     'packname'	=> "libxdmcp6",
     'path'	=> "xorg/lib",
     'name'	=> "libXdmcp",
     'version'	=> "1.0.2",
     'tag'	=> "libXdmcp-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libfontenc",
     'packname'	=> "libfontenc1",
     'path'	=> "xorg/lib",
     'name'	=> "libfontenc",
     'version'	=> "1.0.4",
     'tag'	=> "libfontenc-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "liblbxutil",
     'packname'	=> "liblbxutil1",
     'path'	=> "xorg/lib",
     'name'	=> "liblbxutil",
     'version'	=> "1.0.1",
     'tag'	=> "liblbxutil-1_0_1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libpciaccess",
     #### XXXX correct?
     'packname'	=> "libpciaccess1",
     'path'	=> "xorg/lib",
     'name'	=> "libpciaccess",
     #### XXX Not uptodate in cooker?
     'version'	=> "0.9.1",
     'tag'	=> "libpciaccess-0.9.1",
     'skip'	=> "ignore",
 },
 {
     # Candidate to drop?
     'package'	=> "liboldx",
     'packname'	=> "liboldx6",
     'path'	=> "xorg/lib",
     'name'	=> "liboldX",
     'version'	=> "1.0.1",
     ## no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libx11",
     'packname'	=> "libx11_6",
     'path'	=> "xorg/lib",
     'name'	=> "libX11",
     'version'	=> "1.1.3",
     'tag'	=> "libX11-1.1.3",
     'skip'	=> "ignore",
 },
 {
     ### XXX needs update in distro?
     'package'	=> "libxfont",
     'packname'	=> "libxfont1",
     'path'	=> "xorg/lib",
     'name'	=> "libXfont",
     'version'	=> "1.3.1",
     'tag'	=> "libXfont-1.3.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libsm",
     'packname'	=> "libsm6",
     'path'	=> "xorg/lib",
     'name'	=> "libSM",
     'version'	=> "1.0.3",
     'tag'	=> "libSM-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxt",
     'packname'	=> "libxt6",
     'path'	=> "xorg/lib",
     'name'	=> "libXt",
     'version'	=> "1.0.2",
     'tag'	=> "libXt-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libdmx",
     'packname'	=> "libdmx1",
     'path'	=> "xorg/lib",
     'name'	=> "libdmx",
     'version'	=> "1.0.2",
     ## no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libapplewm",
     'packname'	=> "libapplewm7",
     'path'	=> "xorg/lib",
     'name'	=> "libAppleWM",
     'version'	=> "1.0.0",
     ## no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxv",
     'packname'	=> "libxv1",
     'path'	=> "xorg/lib",
     'name'	=> "libXv",
     'version'	=> "1.0.3",
     'tag'	=> "libXv-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxscrnsaver",
     'packname'	=> "libxscrnsaver1",
     'path'	=> "xorg/lib",
     'name'	=> "libXScrnSaver",
     'version'	=> "1.1.2",
     'tag'	=> "libXScrnSaver-1.1.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxext",
     'packname'	=> "libxext6",
     'path'	=> "xorg/lib",
     'name'	=> "libXext",
     'version'	=> "1.0.3",
     'tag'	=> "libXext-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libwindowswm",
     'packname'	=> "libwindowswm7",
     'path'	=> "xorg/lib",
     'name'	=> "libWindowsWM",
     'version'	=> "1.0.0",
     ## no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxres",
     'packname'	=> "libxres1",
     'path'	=> "xorg/lib",
     'name'	=> "libXRes",
     'version'	=> "1.0.3",
     'tag'	=> "libXres-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxxf86vm",
     'packname'	=> "libxxf86vm1",
     'path'	=> "xorg/lib",
     'name'	=> "libXxf86vm",
     'version'	=> "1.0.1",
     'tag'	=> "libXxf86vm-1_0_1",
     'skip'	=> "ignore",
 },
 {
     ### XXX version 4 and packname ending in 3, correct?
     'package'	=> "libxfixes",
     'packname'	=> "libxfixes3",
     'path'	=> "xorg/lib",
     'name'	=> "libXext",
     'version'	=> "4.0.3",
     'tag'	=> "libXfixes-4.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxinerama",
     'packname'	=> "libxinerama1",
     'path'	=> "xorg/lib",
     'name'	=> "libXinerama",
     'version'	=> "1.0.2",
     'tag'	=> "libXinerama-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxtst",
     'packname'	=> "libxtst6",
     'path'	=> "xorg/lib",
     'name'	=> "libXtst",
     'version'	=> "1.0.3",
     'tag'	=> "libXtst-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxi",
     'packname'	=> "libxi6",
     'path'	=> "xorg/lib",
     'name'	=> "libXi",
     'version'	=> "1.1.3",
     'tag'	=> "libXi-1.1.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxfontcache",
     'packname'	=> "libxfontcache1",
     'path'	=> "xorg/lib",
     'name'	=> "libXfontcache",
     'version'	=> "1.0.4",
     'tag'	=> "libXfontcache-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxkbfile",
     'packname'	=> "libxkbfile1",
     'path'	=> "xorg/lib",
     'name'	=> "libxkbfile",
     'version'	=> "1.0.4",
     'tag'	=> "libxkbfile-1.0.4",
     'skip'	=> "ignore",
 },
 {
     ## Need update in distro?
     'package'	=> "libxxf86dga",
     'packname'	=> "libxxf86dga1",
     'path'	=> "xorg/lib",
     'name'	=> "libXxf86dga",
     'version'	=> "1.0.2",
     'tag'	=> "libXxf86dga-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxevie",
     'packname'	=> "libxevie1",
     'path'	=> "xorg/lib",
     'name'	=> "libXevie",
     'version'	=> "1.0.2",
     'tag'	=> "libXevie-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxprintutil",
     'packname'	=> "libxprintutil1",
     'path'	=> "xorg/lib",
     'name'	=> "libXprintUtil",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxxf86misc",
     'packname'	=> "libxxf86misc1",
     'path'	=> "xorg/lib",
     'name'	=> "libXxf86misc",
     'version'	=> "1.0.1",
     'tag'	=> "libXxf86misc-1_0_1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxrender",
     'packname'	=> "libxrender1",
     'path'	=> "xorg/lib",
     'name'	=> "libXrender",
     'version'	=> "0.9.4",
     'tag'	=> "libXrender-0.9.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxcursor",
     'packname'	=> "libxcursor1",
     'path'	=> "xorg/lib",
     'name'	=> "libXcursor",
     'version'	=> "1.1.9",
     'tag'	=> "ibXcursor-1.1.9",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxpm",
     'packname'	=> "libxpm4",
     'path'	=> "xorg/lib",
     'name'	=> "libXpm",
     'version'	=> "3.5.7",
     'tag'	=> "libXpm-3.5.7",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxdamage",
     'packname'	=> "libxdamage1",
     'path'	=> "xorg/lib",
     'name'	=> "libXdamage",
     'version'	=> "1.1.1",
     'tag'	=> "libXdamage-1.1.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxcomposite",
     'packname'	=> "libxcomposite1",
     'path'	=> "xorg/lib",
     'name'	=> "libXcomposite",
     'version'	=> "0.4.0",
     'tag'	=> "ibXcomposite-0.4.0",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxft",
     'packname'	=> "libxft2",
     'path'	=> "xorg/lib",
     'name'	=> "libXft",
     'version'	=> "2.1.12",
     'tag'	=> "libXft-2.1.12",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxtrap",
     'packname'	=> "libxtrap6",
     'path'	=> "xorg/lib",
     'name'	=> "libXTrap",
     'version'	=> "1.0.0",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxmu",
     'packname'	=> "libxmu6",
     'path'	=> "xorg/lib",
     'name'	=> "libXmu",
     'version'	=> "1.0.3",
     'tag'	=> "libXmu-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxrandr",
     'packname'	=> "libxrandr2",
     'path'	=> "xorg/lib",
     'name'	=> "libXrandr",
     'version'	=> "1.2.2",
     'tag'	=> "libXrandr-1.2.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxvmc",
     'packname'	=> "libxvmc1",
     'path'	=> "xorg/lib",
     'name'	=> "libXv",
     'version'	=> "1.0.4",
     'tag'	=> "libXvMC-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxprintapputil",
     ### really libxprintapputil1?
     'packname'	=> "libxprintapputil1",
     'path'	=> "xorg/lib",
     'name'	=> "libXprintAppUtil",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxaw",
     # ... libxaw[6-8]
     'packname'	=> "libxaw7",
     'path'	=> "xorg/lib",
     'name'	=> "libXaw",
     'version'	=> "1.0.4",
     'tag'	=> "libXaw-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "libxcalibrate",
     'packname'	=> "libxcalibrate1",
     'path'	=> "xorg/lib",
     'name'	=> "libXCalibrate",
     'version'	=> "0.4.0",
     # No tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "libxkbui",
     'packname'	=> "libxkbui1",
     'path'	=> "xorg/lib",
     'name'	=> "libxkbui",
     'version'	=> "1.0.2",
     'tag'	=> "libxkbui-1_0_2",
     'skip'	=> "ignore",
 },
 {
     # XXX libxlg3d not in distro (xorg/lib/libXlg3d)
     'package'	=> "libxlg3d",
     'packname'	=> "libxlg3d1",
     'path'	=> "xorg/lib",
     'name'	=> "libXlg3d",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "libxp",
     'packname'	=> "libxp6",
     'path'	=> "xorg/lib",
     'name'	=> "libXp",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },

 # libxtrans not availabe in distro???
 # libxxf86rush not availabe in distro???

 {
     'package'	=> "pixman",
     'packname'	=> "libpixman-1_0",
     'path'	=> "",	# empty path
     'name'	=> "pixman",
     'version'	=> "0.9.6",
     'tag'	=> "pixman-0.9.6",
     'skip'	=> "ignore",
 },
 {
     # git-archive --format=tar --prefix=/libdrm-2.3.0/ drm-2.3.0 | bzip2 > libdrm-2.3.0.tar.bz2
     'package'	=> "libdrm",
     'packname'	=> "libdrm2",
     'path'	=> "mesa",
     'name'	=> "drm",
     'version'	=> "2.3.0",
     'tag'	=> "drm-2.3.0",
     'skip'	=> "ignore",
 },
 {
     # git-archive --format=tar --prefix=/mesa-7.0.2/ mesa_7_0_2 | bzip2 > mesa-7.0.2.tar.bz2
     'package'	=> "mesa",
     'path'	=> "mesa",
     'name'	=> "mesa",
     'version'	=> "7.0.2",
     'tag'	=> "mesa_7_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-data-bitmaps",
     'path'	=> "data",
     'name'	=> "bitmaps",
     'version'	=> "1.0.1",
     # no tag available
     'skip'	=> "ignore",
 },
 {
     # XXX what package? x11-data-cursors doesn't exist
     'package'	=> "x11-data-cursors",
     'path'	=> "data",
     'name'	=> "cursors",
     'version'	=> "1.0.1",
     # no tag available
     'skip'	=> "ignore",
 },

 {
     'package'	=> "x11-server",
     'path'	=> "xorg/xserver",
     # should revert to use tag of server 1.4 not use 1.4 branch
     'skip'	=> "yes",
 },

 {
     'package'	=> "x11-driver-input-acecad",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-acecad",
     'version'	=> "1.2.1",
     'tag'	=> "xf86-input-acecad-1.2.1",
 },
 {
     'package'	=> "x11-driver-input-aiptek",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-aiptek",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-aiptek-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-calcomp",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-aiptek",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-aiptek-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-citron",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-citron",
     'version'	=> "2.2.1",
     'tag'	=> "xf86-input-citron-2.2.1",
 },
 {
     'package'	=> "x11-driver-input-digitaledge",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-digitaledge",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-digitaledge-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-dmc",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-dmc",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-dmc-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-dynapro",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-dynapro",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-dynapro-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-elo2300",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-elo2300",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-elo2300-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-elographics",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-elographics",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-elographics-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-evdev",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-devdev",
     'version'	=> "1.2.0",
     'tag'	=> "xf86-input-evdev-1.2.0",
 },
 {
     'package'	=> "x11-driver-input-fpit",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-fpit",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-fpit-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-hyperpen",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-hyperpen",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-hyperpen-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-jamstudio",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-jamstudio",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-jamstudio-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-joystick",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-joystick",
     'version'	=> "1.3.0",
     'tag'	=> "xf86-input-joystick-1.3.0",
 },
 {
     'package'	=> "x11-driver-input-keyboard",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-keyboard",
     'version'	=> "1.2.2",
     'tag'	=> "xf86-input-keyboard-1.2.2",
 },
 {
     'package'	=> "x11-driver-input-magellan",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-magellan",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-magellan-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-magictouch",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-magictouch",
     'version'	=> "1.0.0.5",
     'tag'	=> "xf86-input-magictouch-1.0.0.5\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-microtouch",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-microtouch",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-microtouch-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-mouse",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-mouse",
     'version'	=> "1.2.3",
     'tag'	=> "xf86-input-mouse-1.2.3",
 },
 {
     'package'	=> "x11-driver-input-mutouch",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-mutouch",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-mutouch-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-palmax",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-palmax",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-palmax-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-penmount",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-penmount",
     'version'	=> "1.2.1",
     'tag'	=> "xf86-input-penmount-1.2.1",
 },
 # x11-driver-input-sample unbuildable? or can be added?
 {
     'package'	=> "x11-driver-input-spaceorb",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-spaceorb",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-spaceorb-1.1.1",
 },
 {
     'package'	=> "x11-driver-input-summa",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-summa",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-summa-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-tek4957",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-tek4957",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-tek4957-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-ur98",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-ur98",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-ur98-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-vmmouse",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-vmmouse",
     'version'	=> "12.4.3",
     'tag'	=> "xf86-input-vmmouse-12.4.3\@mandriva",
 },
 {
     'package'	=> "x11-driver-input-void",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-input-void",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-void-1.1.1",
 },
 {
     'package'	=> "x11-driver-video-amd",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-amd",
     'version'	=> "2.7.7.3",
     'tag'	=> "xf86-video-amd-2.7.7.3",
 },
 {
     'package'	=> "x11-driver-video-apm",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-apm",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-video-apm-1.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-ark",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-ark",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-video-ark-1.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-ast",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-ast",
     'version'	=> "0.81.0.0",
     'tag'	=> "xf86-video-ast-0.81.0.0\@mandriva",
 },

 {
     # Needs merge with newest version 6.7.197
     'package'	=> "x11-driver-video-ati",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-ati",
     'version'	=> "6.7.196",
     'tag'	=> "xf86-video-ati-6.7.196",
 },

 {
     'package'	=> "x11-driver-video-chips",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-chips",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-chips-1.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-cirrus",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-cirrus",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-input-cirrus-1.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-cyrix",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-cyrix",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-input-cyrix-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-dummy",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-dummy",
     'version'	=> "0.2.0",
     'tag'	=> "xf86-input-cyrix-0.2.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-fbdev",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-fbdev",
     'version'	=> "0.3.1",
     'tag'	=> "xf86-video-fbdev-0.3.1",
 },

 # xf86-video-glide not available - consider adding a glide driver

 {
     'package'	=> "x11-driver-video-glint",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-glint",
     'version'	=> "1.1.1",
     # XXXX Tag @mandriva not done or not pushed to git.mandriva.com
     'tag'	=> "glint-1_1_1",
 },

 {
     'package'	=> "x11-driver-video-i128",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-i128",
     'version'	=> "1.2.1",
     'tag'	=> "xf86-video-i128-1.2.1",
 },
 {
     'package'	=> "x11-driver-video-i740",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-i740",
     'version'	=> "1.1.0",
     # XXX Tag @mandriva not done or not pushed
     'tag'	=> "i740-1_1_0",
 },
 {
     'package'	=> "x11-driver-video-impact",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-impact",
     'version'	=> "0.2.0",
     'tag'	=> "xf86-video-impact-0.2.0\@mandriva",
 },

 {
     'package'	=> "x11-driver-video-imstt",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-imstt",
     'version'	=> "1.1.0",
     # XXX Tag @mandriva not done or not pushed
     'tag'	=> "imstt-1_1_0",
 },
 {
     'package'	=> "x11-driver-video-intel",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-intel",
     'version'	=> "2.2.0",
     'tag'	=> "xf86-video-intel-2.2.0",
 },
 {
     'package'	=> "x11-driver-video-mga",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-mga",
     'version'	=> "1.4.7",
     'tag'	=> "xf86-video-mga-1.4.7",
 },
 {
     'package'	=> "x11-driver-video-neomagic",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-neomagic",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-video-neomagic-1.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-newport",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-newport",
     'version'	=> "0.2.1",
     'tag'	=> "xf86-video-newport-0.2.1",
 },
 {
     'package'	=> "x11-driver-video-nsc",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-nsc",
     'version'	=> "2.8.3",
     'tag'	=> "xf86-video-nsc-2.8.3",
 },
 {
     'package'	=> "x11-driver-video-nv",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-nv",
     'version'	=> "2.1.6",
     'tag'	=> "xf86-video-nv-2.1.6\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-radeonhd",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-radeonhd",
     'version'	=> "20071214",
     # No tags available
 },
 {
     'package'	=> "x11-driver-video-rendition",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-rendition",
     'version'	=> "4.1.3",
     'tag'	=> "xf86-video-rendition-4.1.3",
 },
 {
     'package'	=> "x11-driver-video-s3",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-s3",
     'version'	=> "0.5.0",
     'tag'	=> "xf86-video-s3-0.5.0",
 },
 {
     'package'	=> "x11-driver-video-s3virge",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-s3virge",
     'version'	=> "1.9.1",
     # Tag @mandriva exists and should be removed
     'tag'	=> "xf86-video-s3virge-1.9.1",
 },
 {
     'package'	=> "x11-driver-video-savage",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-savage",
     'version'	=> "2.1.3",
     'tag'	=> "xf86-video-savage-2.1.3",
 },
 {
     'package'	=> "x11-driver-video-siliconmotion",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-siliconmotion",
     'version'	=> "1.5.1",
     'tag'	=> "xf86-video-siliconmotion-1.5.1",
 },
 {
     'package'	=> "x11-driver-video-sis",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-sis",
     'version'	=> "0.9.4",
     'tag'	=> "xf86-video-sis-0.9.4",
 },
 {
     'package'	=> "x11-driver-video-sisusb",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-sisusb",
     'version'	=> "0.8.1",
     'tag'	=> "xf86-video-sisusb-0.8.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-sunbw2",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-sunbw2",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-sunbw2-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-suncg14",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-suncg14",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-suncg14-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-suncg3",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-suncg3",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-suncg3-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-suncg6",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-suncg6",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-suncg6-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-sunffb",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-sunffb",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-sunffb-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-sunleo",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-sunleo",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-sunleo-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-suntcx",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-suntcx",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-suntcx-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-tdfx",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-tdfx",
     'version'	=> "1.3.0",
 },
 {
     'package'	=> "x11-driver-video-tga",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-tga",
     'version'	=> "1.1.0",
     'tag'	=> "xf86-video-tga-1.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-trident",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-trident",
     'version'	=> "1.2.3",
     'tag'	=> "xf86-video-trident-1.2.3",
 },
 {
     'package'	=> "x11-driver-video-tseng",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-tseng",
     'version'	=> "1.1.1",
 },
 {
     'package'	=> "x11-driver-video-v4l",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-v4l",
     'version'	=> "0.1.1",
     'tag'	=> "xf86-video-v4l-0.1.1\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-vesa",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-vesa",
     'version'	=> "1.3.0",
     'tag'	=> "xf86-video-vesa-1.3.0",
 },
 {
     'package'	=> "x11-driver-video-vga",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-vga",
     'version'	=> "4.1.0",
     'tag'	=> "xf86-video-vga-4.1.0\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-via",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-via",
     'version'	=> "0.2.2",
     'tag'	=> "xf86-video-via-0.2.2",
 },
 {
     'package'	=> "x11-driver-video-vmware",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-vmware",
     'version'	=> "10.15.2",
     'tag'	=> "xf86-video-vmware-10.15.2\@mandriva",
 },
 {
     'package'	=> "x11-driver-video-voodoo",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-voodoo",
     'version'	=> "1.1.1",
     'tag'	=> "xf86-video-voodoo-1.1.1",
 },
 #  xf86-video-wsfb is BSD only
 {
     'package'	=> "x11-driver-video-xgi",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-xgi",
     'version'	=> "1.5.0",
     'tag'	=> "xf86-video-xgi-1.5.0",
 },
 {
     'package'	=> "x11-driver-video-xgixp",
     'path'	=> "xorg/driver",
     'name'	=> "xf86-video-xgi",
     'version'	=> "1.7.99.2",
     # No tags available and not buildable as needs current/master git bits
     # as well as kernel sources that may not yet be in upstream/stable
 },
 {
     'package'	=> "x11-driver-video-avivo",
     'path'	=> "avivo",
     'name'	=> "xf86-video-avivo",
     'version'	=> "0.0.1",
     'tag'	=> "xf86-video-avivo-0.0.1",
 },
 {
     'package'	=> "x11-driver-video-nouveau",
     'path'	=> "nouveau",
     'name'	=> "xf86-video-nouveau",
     'version'	=> "1.2.0",
     # No tags available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "x11-font-alias",
     'path'	=> "xorg/font",
     'name'	=> "alias",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-encodings",
     'path'	=> "xorg/font",
     'name'	=> "encodings",
     'version'	=> "1.0.2",
     'tag'	=> "encodings-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-util",
     'path'	=> "xorg/font",
     'name'	=> "util",
     'version'	=> "1.0.1",
     'tag'	=> "font-util-1_0_1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-adobe-100dpi",
     'path'	=> "xorg/font",
     'name'	=> "adobe-100dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-adobe-75dpi",
     'path'	=> "xorg/font",
     'name'	=> "adobe-75dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-adobe-utopia-100dpi",
     'path'	=> "xorg/font",
     'name'	=> "adobe-utopia-100dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-adobe-utopia-75dpi",
     'path'	=> "xorg/font",
     'name'	=> "adobe-utopia-75dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-arabic-misc",
     'path'	=> "xorg/font",
     'name'	=> "arabic-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-100dpi",
     'path'	=> "xorg/font",
     'name'	=> "bh-100dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-75dpi",
     'path'	=> "xorg/font",
     'name'	=> "bh-75dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-lucidatypewriter-100dpi",
     'path'	=> "xorg/font",
     'name'	=> "bh-lucidatypewriter-100dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-lucidatypewriter-75dpi",
     'path'	=> "xorg/font",
     'name'	=> "bh-lucidatypewriter-75dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-ttf",
     'path'	=> "xorg/font",
     'name'	=> "bh-ttf",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-type1",
     'path'	=> "xorg/font",
     'name'	=> "bh-type1",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bh-type1",
     'path'	=> "xorg/font",
     'name'	=> "bh-type1",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bitstream-100dpi",
     'path'	=> "xorg/font",
     'name'	=> "bitstream-100dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bitstream-75dpi",
     'path'	=> "xorg/font",
     'name'	=> "bitstream-75dpi",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bitstream-speedo",
     'path'	=> "xorg/font",
     'name'	=> "bitstream-speedo",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-bitstream-type1",
     'path'	=> "xorg/font",
     'name'	=> "bitstream-type1",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-cronyx-cyrillic",
     'path'	=> "xorg/font",
     'name'	=> "cronyx-cyrillic",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-cursor-misc",
     'path'	=> "xorg/font",
     'name'	=> "cursor-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-daewoo-misc",
     'path'	=> "xorg/font",
     'name'	=> "daewoo-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-dec-misc",
     'path'	=> "xorg/font",
     'name'	=> "dec-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-ibm-type1",
     'path'	=> "xorg/font",
     'name'	=> "ibm-type1",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-isas-misc",
     'path'	=> "xorg/font",
     'name'	=> "isas-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-jis-misc",
     'path'	=> "xorg/font",
     'name'	=> "jis-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-micro-misc",
     'path'	=> "xorg/font",
     'name'	=> "micro-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-misc-cyrillic",
     'path'	=> "xorg/font",
     'name'	=> "misc-cyrillic",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-misc-ethiopic",
     'path'	=> "xorg/font",
     'name'	=> "misc-ethiopic",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-misc-meltho",
     'path'	=> "xorg/font",
     'name'	=> "misc-meltho",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-misc-misc",
     'path'	=> "xorg/font",
     'name'	=> "misc-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-schumacher-misc",
     'path'	=> "xorg/font",
     'name'	=> "schumacher-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-screen-cyrillic",
     'path'	=> "xorg/font",
     'name'	=> "screen-cyrillic",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-sony-misc",
     'path'	=> "xorg/font",
     'name'	=> "sony-misc",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-winitzki-cyrillic",
     'path'	=> "xorg/font",
     'name'	=> "winitzki-cyrillic",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11-font-xfree86-type1",
     'path'	=> "xorg/font",
     'name'	=> "xfree86-type1",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "appres",
     'path'	=> "xorg/app",
     'name'	=> "appres",
     'version'	=> "1.0.1",
     'tag'	=> "appres-1.0.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "bdftopcf",
     'path'	=> "xorg/app",
     'name'	=> "bdftopcf",
     'version'	=> "1.0.1",
     'tag'	=> "bdftopcf-1.0.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "beforelight",
     'path'	=> "xorg/app",
     'name'	=> "beforelight",
     'version'	=> "1.0.3",
     'tag'	=> "beforelight-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "bitmap",
     'path'	=> "xorg/app",
     'name'	=> "bitmap",
     'version'	=> "1.0.3",
     'tag'	=> "beforelight-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "compiz",
     'path'	=> "xorg/app",
     'name'	=> "compiz",
     'version'	=> "0.6.2",
     'tag'	=> "compiz-0.6.2",
     'skip'	=> "ignore",
 },
 # XXX Sun console type detector, just ignore package?
 {
     'package'	=> "constype",
     'path'	=> "xorg/app",
     'name'	=> "constype",
     'version'	=> "1.0.1",
     'tag'	=> "constype-1.0.1",
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 # Maybe fix this package?
 {
     'package'	=> "edid-decode",
     'path'	=> "xorg/app",
     'name'	=> "edid-decode",
     # No version or tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "editres",
     'path'	=> "xorg/app",
     'name'	=> "editres",
     'version'	=> "1.0.3",
     'tag'	=> "editres-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "fdclock",
     'path'	=> "xorg/app",
     'name'	=> "fdclock",
     # No version or tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "fonttosfnt",
     'path'	=> "xorg/app",
     'name'	=> "fonttosfnt",
     'version'	=> "1.0.3",
     'tag'	=> "fonttosfnt-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "fslsfonts",
     'path'	=> "xorg/app",
     'name'	=> "fslsfonts",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "fstobdf",
     'path'	=> "xorg/app",
     'name'	=> "fstobdf",
     'version'	=> "1.0.2",
     'tag'	=> "fstobdf-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "glxcompmgr",
     'path'	=> "xorg/app",
     'name'	=> "glxcompmgr",
     # No version or tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "grandr",
     'path'	=> "xorg/app",
     'name'	=> "grandr",
     'version'	=> "0.1",
     'tag'	=> "grandr-0.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "iceauth",
     'path'	=> "xorg/app",
     'name'	=> "iceauth",
     'version'	=> "1.0.2",
     'tag'	=> "iceauth-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "ico",
     'path'	=> "xorg/app",
     'name'	=> "ico",
     'version'	=> "1.0.2",
     'tag'	=> "ico-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "intel-gen4asm",
     'path'	=> "xorg/app",
     'name'	=> "intel-gen4asm",
     # No version or tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "lbxproxy",
     'path'	=> "xorg/app",
     'name'	=> "lbxproxy",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "listres",
     'path'	=> "xorg/app",
     'name'	=> "listres",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "luit",
     'path'	=> "xorg/app",
     'name'	=> "luit",
     'version'	=> "1.0.2",
     'tag'	=> "luit-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "makepsres",
     'path'	=> "xorg/app",
     'name'	=> "makepsres",
     # No version, tag or build available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "mkcfm",
     'path'	=> "xorg/app",
     'name'	=> "mkcfm",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
     'BROKEN'	=> "missing symbols CIDOpenScalable, CIDRegisterFontFileFunctions (should be in libXfont ?)"
 },
 {
     'package'	=> "mkcomposecache",
     'path'	=> "xorg/app",
     'name'	=> "mkcomposecache",
     'version'	=> "1.2",
     'tag'	=> "xorg-app-mkcomposecache-1.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "mkfontdir",
     'path'	=> "xorg/app",
     'name'	=> "mkfontdir",
     'version'	=> "1.0.3",
     # Only a tag for 1.0.2 available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "mkfontscale",
     'path'	=> "xorg/app",
     'name'	=> "mkfontscale",
     'version'	=> "1.0.3",
     'tag'	=> "mkfontscale-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "oclock",
     'path'	=> "xorg/app",
     'name'	=> "oclock",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "pclcomp",
     'path'	=> "xorg/app",
     'name'	=> "pclcomp",
     # No version, tag or any file available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "proxymngr",
     'path'	=> "xorg/app",
     'name'	=> "proxymngr",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "rendercheck",
     'path'	=> "xorg/app",
     'name'	=> "rendercheck",
     'version'	=> "1.2",
     'tag'	=> "rendercheck-1.2",
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "rgb",
     'path'	=> "xorg/app",
     'name'	=> "rgb",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "rstart",
     'path'	=> "xorg/app",
     'name'	=> "rstart",
     'version'	=> "1.0.2",
     'tag'	=> "rstart-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "scripts",
     'path'	=> "xorg/app",
     'name'	=> "scripts",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     'package'	=> "sessreg",
     'path'	=> "xorg/app",
     'name'	=> "sessreg",
     'version'	=> "1.0.3",
     'tag'	=> "sessreg-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "setxkbmap",
     'path'	=> "xorg/app",
     'name'	=> "setxkbmap",
     'version'	=> "1.0.4",
     'tag'	=> "setxkbmap-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "showfont",
     'path'	=> "xorg/app",
     'name'	=> "showfont",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "twm",
     'path'	=> "xorg/app",
     'name'	=> "twm",
     'version'	=> "1.0.3",
     'tag'	=> "twm-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "viewres",
     'path'	=> "xorg/app",
     'name'	=> "viewres",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "x11perf",
     'path'	=> "xorg/app",
     'name'	=> "x11perf",
     'version'	=> "1.4.1",
     'tag'	=> "x11perf-1_4_1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xauth",
     'path'	=> "xorg/app",
     'name'	=> "xauth",
     'version'	=> "1.0.2",
     'tag'	=> "xauth-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xbacklight",
     'path'	=> "xorg/app",
     'name'	=> "xbacklight",
     'version'	=> "1.1",
     'tag'	=> "xbacklight-1.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xbiff",
     'path'	=> "xorg/app",
     'name'	=> "xbiff",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xcalc",
     'path'	=> "xorg/app",
     'name'	=> "xcalc",
     'version'	=> "1.0.2",
     'tag'	=> "xcalc-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xclipboard",
     'path'	=> "xorg/app",
     'name'	=> "xclipboard",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xclock",
     'path'	=> "xorg/app",
     'name'	=> "xclock",
     'version'	=> "1.0.3",
     'tag'	=> "xclock-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xcmsdb",
     'path'	=> "xorg/app",
     'name'	=> "xcmsdb",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xcompmgr",
     'path'	=> "xorg/app",
     'name'	=> "xcompmgr",
     'version'	=> "1.1.3",
     'tag'	=> "XCOMPMGR_1_1_3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xconsole",
     'path'	=> "xorg/app",
     'name'	=> "xconsole",
     'version'	=> "1.0.3",
     'tag'	=> "xconsole-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xcursorgen",
     'path'	=> "xorg/app",
     'name'	=> "xcursorgen",
     'version'	=> "1.0.2",
     'tag'	=> "xcursorgen-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xdbedizzy",
     'path'	=> "xorg/app",
     'name'	=> "xdbedizzy",
     'version'	=> "1.0.2",
     'tag'	=> "xdbedizzy-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xditview",
     'path'	=> "xorg/app",
     'name'	=> "xditview",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xdm",
     'path'	=> "xorg/app",
     'name'	=> "xdm",
     'version'	=> "1.1.6",
     'tag'	=> "xdm-1.1.6",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xdpyinfo",
     'path'	=> "xorg/app",
     'name'	=> "xdpyinfo",
     'version'	=> "1.0.2",
     'tag'	=> "xdpyinfo-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xdriinfo",
     'path'	=> "xorg/app",
     'name'	=> "xdriinfo",
     'version'	=> "1.0.2",
     'tag'	=> "xdriinfo-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xedit",
     'path'	=> "xorg/app",
     'name'	=> "xedit",
     'version'	=> "1.0.2",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xev",
     'path'	=> "xorg/app",
     'name'	=> "xev",
     'version'	=> "1.0.2",
     'tag'	=> "xev-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xeyes",
     'path'	=> "xorg/app",
     'name'	=> "xeyes",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xf86dga",
     'path'	=> "xorg/app",
     'name'	=> "xf86dga",
     'version'	=> "1.0.2",
     'tag'	=> "xf86dga-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfd",
     'path'	=> "xorg/app",
     'name'	=> "xfd",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfindproxy",
     'path'	=> "xorg/app",
     'name'	=> "xfindproxy",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfontsel",
     'path'	=> "xorg/app",
     'name'	=> "xfontsel",
     'version'	=> "1.0.2",
     'tag'	=> "xfontsel-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfs",
     'path'	=> "xorg/app",
     'name'	=> "xfs",
     'version'	=> "1.0.5",
     'tag'	=> "xfs-1.0.5",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfsinfo",
     'path'	=> "xorg/app",
     'name'	=> "xfsinfo",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xfwp",
     'path'	=> "xorg/app",
     'name'	=> "xfwp",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xgamma",
     'path'	=> "xorg/app",
     'name'	=> "xgamma",
     'version'	=> "1.0.2",
     'tag'	=> "xgamma-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xgc",
     'path'	=> "xorg/app",
     'name'	=> "xgc",
     'version'	=> "1.0.2",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xhost",
     'path'	=> "xorg/app",
     'name'	=> "xhost",
     'version'	=> "1.0.2",
     'tag'	=> "xhost-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xinit",
     'path'	=> "xorg/app",
     'name'	=> "xinit",
     'version'	=> "1.0.7",
     'tag'	=> "xhost-1.0.7",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xkbcomp",
     'path'	=> "xorg/app",
     'name'	=> "xkbcomp",
     'version'	=> "1.0.3",
     'tag'	=> "xkbcomp-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xkbevd",
     'path'	=> "xorg/app",
     'name'	=> "xkbevd",
     'version'	=> "1.0.2",
     'tag'	=> "xkbevd-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xkbprint",
     'path'	=> "xorg/app",
     'name'	=> "xkbprint",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xkbutils",
     'path'	=> "xorg/app",
     'name'	=> "xkbutils",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xkill",
     'path'	=> "xorg/app",
     'name'	=> "xkill",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xload",
     'path'	=> "xorg/app",
     'name'	=> "xload",
     'version'	=> "1.0.2",
     'tag'	=> "xload-1.0.2",
     'skip'	=> "ignore",
 },
 {
     ## XXX has BuildRequires of libxprintutil-devel
     'package'	=> "xlogo",
     'path'	=> "xorg/app",
     'name'	=> "xlogo",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xlsatoms",
     'path'	=> "xorg/app",
     'name'	=> "xlsatoms",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xlsclients",
     'path'	=> "xorg/app",
     'name'	=> "xlsclients",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xlsfonts",
     'path'	=> "xorg/app",
     'name'	=> "xlsfonts",
     'version'	=> "1.0.2",
     'tag'	=> "xlsfonts-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xmag",
     'path'	=> "xorg/app",
     'name'	=> "xmag",
     'version'	=> "1.0.2",
     'tag'	=> "xmag-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xman",
     'path'	=> "xorg/app",
     'name'	=> "xman",
     'version'	=> "1.0.3",
     'tag'	=> "xman-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xmessage",
     'path'	=> "xorg/app",
     'name'	=> "xmessage",
     'version'	=> "1.0.2",
     'tag'	=> "xmessage-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xmh",
     'path'	=> "xorg/app",
     'name'	=> "xmh",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xmodmap",
     'path'	=> "xorg/app",
     'name'	=> "xmodmap",
     'version'	=> "1.0.3",
     'tag'	=> "xmodmap-1.0.3",
     'skip'	=> "ignore",
 },
 {
     ## XXX has BuildRequires of libxprintutil-devel, but is known to not
     ## work without xprint support
     'package'	=> "xmore",
     'path'	=> "xorg/app",
     'name'	=> "xmore",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     ## XXX BuildRequires: libxprintapputil-devel libxprintutil-devel
     'package'	=> "xphelloworld",
     'path'	=> "xorg/app",
     'name'	=> "xphelloworld",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     ## XXX BuildRequires: libxprintapputil-devel libxprintutil-devel
     'package'	=> "xplsprinters",
     'path'	=> "xorg/app",
     'name'	=> "xplsprinters",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xpr",
     'path'	=> "xorg/app",
     'name'	=> "xpr",
     'version'	=> "1.0.2",
     'tag'	=> "xpr-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xprehashprinterlist",
     'path'	=> "xorg/app",
     'name'	=> "xprehashprinterlist",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xprop",
     'path'	=> "xorg/app",
     'name'	=> "xprop",
     'version'	=> "1.0.3",
     'tag'	=> "xprop-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xrandr",
     'path'	=> "xorg/app",
     'name'	=> "xrandr",
     'version'	=> "1.2.2",
     'tag'	=> "xrandr-1.2.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xrdb",
     'path'	=> "xorg/app",
     'name'	=> "xrdb",
     'version'	=> "1.0.4",
      'tag'	=> "xrdb-1.0.4",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xrefresh",
     'path'	=> "xorg/app",
     'name'	=> "xrefresh",
     'version'	=> "1.0.2",
     'tag'	=> "xrefresh-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xrestop",
     'path'	=> "xorg/app",
     'name'	=> "xrestop",
     'version'	=> "0.4",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xrx",
     'path'	=> "xorg/app",
     'name'	=> "xrx",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xset",
     'path'	=> "xorg/app",
     'name'	=> "xset",
     'version'	=> "1.0.3",
     'tag'	=> "xset-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xsetmode",
     'path'	=> "xorg/app",
     'name'	=> "xsetmode",
     'version'	=> "1.0.0",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xsetpointer",
     'path'	=> "xorg/app",
     'name'	=> "xsetpointer",
     'version'	=> "1.0.1",
     'tag'	=> "xsetpointer-1.0.1",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xsetroot",
     'path'	=> "xorg/app",
     'name'	=> "xsetroot",
     'version'	=> "1.0.2",
     'tag'	=> "xsetroot-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xshowdamage",
     'path'	=> "xorg/app",
     'name'	=> "xshowdamage",
     # No version or tag available
     'skip'	=> "ignore",
     'TODO'	=> "yes",
 },
 {
     ## Broken has build dependency on rsh (and runtime dep also)
     'package'	=> "xsm",
     'path'	=> "xorg/app",
     'name'	=> "xsm",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xstdcmap",
     'path'	=> "xorg/app",
     'name'	=> "xstdcmap",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xtrap",
     'path'	=> "xorg/app",
     'name'	=> "xtrap",
     'version'	=> "1.0.2",
     'tag'	=> "xtrap-1_0_2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xvidtune",
     'path'	=> "xorg/app",
     'name'	=> "xvidtune",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xvinfo",
     'path'	=> "xorg/app",
     'name'	=> "xvinfo",
     'version'	=> "1.0.2",
     'tag'	=> "xvinfo-1.0.2",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xwd",
     'path'	=> "xorg/app",
     'name'	=> "xwd",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xwininfo",
     'path'	=> "xorg/app",
     'name'	=> "xwininfo",
     'version'	=> "1.0.3",
     'tag'	=> "xwininfo-1.0.3",
     'skip'	=> "ignore",
 },
 {
     'package'	=> "xwud",
     'path'	=> "xorg/app",
     'name'	=> "xwud",
     'version'	=> "1.0.1",
     # No tag available
     'skip'	=> "ignore",
 },
 );



##############################################################################
# Simple wrapper that prints errno string if not successful
sub x_system {
    my ($command) = @_;

    print("    $command\n");
    system($command);
    print(STDERR " ** $!\n") if ($?);
};


##############################################################################
# Simple wrapper that prints errno string if not successful
sub x_chdir {
    my ($dir) = @_;

    print("    cd $dir\n") if ($options{'v'});
    chdir($dir) or die(" ** $!\n");
};


##############################################################################
# Create required files for use in Mandriva buildsystem
sub make_git_dist {
    my ($pack, $path, $name, $version, $prefix, $tag) = @_;

    print("--> $pack\n");

    unless ($version) {
	print(" ** No version specified. Nothing done.\n");
	return;
    }

    # if requires a different format
    $prefix = "$name-$version" unless($prefix);

    # Use default tag format unless a different tag is used
    $tag = "$name-$version" unless ($tag);

    my $full_path = "$x_gitdir/$path/$name";
    x_chdir($full_path);
    print("    git-archive --format=tar --prefix=$prefix/ $tag |",
	  " $compress > $name-$version.tar$compress_ext\n");
    print("    git-format-patch $tag..mandriva+gpl\n");
};


##############################################################################
# Updates git and svn chekout
sub git_update {
    my ($pack, $path, $name) = @_;

    my ($full_path, $update);

    print("--> $pack\n");

    # git bits
    $full_path = "$x_gitdir/$path/$name";
    if (-d $full_path) {
	x_chdir($full_path);
	if (`git diff`) {
	    die("$pack has uncommited changes.\n");
	}
	my $tmp = $/;
	$/ = undef;
	$update = `git branch`;
	$/ = $tmp;
	# Don't try to switch branches, as it may cause conflicting merges.
	# Require that the package leave it in master branch, i.e. just
	# in a state matching a standard/default readonly checkout.
	if ($update !~ /^\* master$/m) {
	    die("$pack is not in branch master.\n");
	}
	$update = "git pull";
    }
    else {
	$full_path = $x_gitdir;
	x_chdir($full_path);
	$update = "git clone $git_protocol://$git_repository/$path/$name $path/$name";
    }
    x_system("$update");

    # rpm bits
    $full_path = "$x_rpmdir/$pack";
    if (-d $full_path) {
	$update = $rpm_update;
    }
    else {
	$full_path = $x_rpmdir;
	$update = $rpm_checkout;
    }
    x_chdir($full_path);
    $update =~ s/%n/$pack/g;
    x_system("$update");
};


##############################################################################
sub git_check {
    my ($pack, $name) = @_;

    my $depsfile = "$x_depsdir/$name.deps";

    print("--> $pack\n");

    if (! -f "$depsfile") {
	print(" ** File $depsfile not found.\n");
	return;
    }

    x_system("x-check-deps.pl -c -d $depsfile");
};


##############################################################################
sub check_deps {
    my ($pack, $name, $version, $prefix) = @_;

    print("--> $pack\n");

    # if requires a different format
    $prefix = "$name-$version" unless($prefix);

    x_chdir("$x_rpmdir/$pack/BUILD/$prefix");
    x_system("x-check-deps.pl -n $name");
};


##############################################################################
sub build {
    my ($pack, $packname, $name, $version, $prefix) = @_;

    print("--> $pack\n");

    # if requires a different format
    $prefix = "$name-$version" unless($prefix);

    my $full_path = "$x_rpmdir/$pack/BUILD/$prefix";

    # Could also use a short-circuit
    if (-d $full_path and -f "$full_path/Makefile") {
	x_chdir($full_path);
	x_system("make");
    }
    else {
	x_chdir("$x_rpmdir/$pack/SPECS");
	x_system("bm -l");
    }

    die(" ** $!\n") if ($?);

    # Check if continuing now will use the proper installed sdk/headers/binaries
    my $installed;
    if ($packname) {
	$installed = `rpm -q $packname`;
	chomp($installed);
	if ($installed eq "") {
	    die(" ** rpm $packname must be installed to continue.\n");
	}
    }
    else {
	$installed = `rpm -q $pack`;
	chomp($installed);
	if ($installed eq "" or $installed !~ /^$pack-$version/) {
	    die(" ** rpm $pack-$version must be installed to continue.\n");
	}
    }
};

##############################################################################
# Subroutines are free to change this now as a lot of "system" commands
# may be used.
my $dir = Cwd::getcwd();

if ($command eq "up") {
    foreach (@modules) {
	git_update($_->{'package'},
		   $_->{'path'},
		   $_->{'name'}) unless($_->{'skip'});
    }
}
elsif ($command eq "dist") {
    foreach (@modules) {
	make_git_dist($_->{'package'},
		      $_->{'path'},
		      $_->{'name'},
		      $_->{'version'},
		      $_->{'prefix'},
		      $_->{'tag'}) unless($_->{'skip'});
    }
}
elsif ($command eq "checkdeps") {
    foreach (@modules) {
	git_check($_->{'package'},
		  $_->{'name'}) unless($_->{'skip'});
    }
}
elsif ($command eq "builddeps") {
    foreach (@modules) {
	check_deps($_->{'package'},
		   $_->{'name'},
		   $_->{'version'},
		   $_->{'prefix'}) unless($_->{'skip'});
    }
}
elsif ($command eq "build") {
    print(" ** NOTE: make sure the latest x11-proto-devel package is installed.\n");
    foreach (@modules) {
	build($_->{'package'},
	      $_->{'packname'},
	      $_->{'name'},
	      $_->{'version'},
	      $_->{'prefix'}) unless($_->{'skip'});
	die(" ** rpm " . $_->{'package'} . " must be buildable to continue.")
	    if ($_->{'skip'} and $_->{'skip'} ne "ignore");
    }
}

# Restore current directory
chdir($dir);
