#!/usr/bin/perl

##############################################################################
#  Copyright (C) 2007 Paulo Cesar Pereira de Andrade.  All Rights Reserved.
#
#  This is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This software is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  Authors:
#	Paulo Cesar Pereira de Andrade <pcpa@mandriva.com.br>
##############################################################################

# This file searches all dependencies in a build tree and creates a file
# named <package>.deps where the contents are:
#	<dependency>:<mtime>
# <dependency> is usually a system header file name
# <mtime> is usually the modification time of the file
#
# Note that it is expected that file modification time doesn't change
# when rebuilding a package, and not changing file contents.


use Cwd;
use File::Basename;
use File::stat;
use Getopt::Std;

use strict;

my %directories;
my %dependencies;
my %olddeps;
my %pkgdeps;
my %options;

my $package_system = "rpm";


##############################################################################
# Check for command line options
# TODO add possibly risky option of not creating dependency of libc and
# gcc header files, and possibly others; the X Protocol for example
# isn't expected to change, but some structures, usually extensions still can...
Getopt::Std::getopts("pd:csfh?", \%options);
if ($options{'h'} or $options{'?'}) {
    print("Usage:\n",
	  File::Basename::basename($0), " [options ...]\n",
	  "-p path\tUse given path as root build dir [default: .]\n",
	  "-d deps\tDependency file to compare agains't.\n",
	  "-n name\tName for new dependency file [default: name of base directory]",
	  "-c Check mode. Doesn't create a new deps file (Needs option -d).\n",
	  "-s\tSilent mode.\n",
	  "-f\tFast mode, doesn't query rpm for package information.\n",
	  "-h, -?\tPrints this message and quits.\n");
    exit(0);
}


##############################################################################
# Check root build dir
$options{'p'} = "." unless $options{'p'};
$options{'p'} = Cwd::realpath(".");
$options{'n'} = File::Basename::basename($options{'p'}) unless $options{'n'};


##############################################################################
# Find autotools dependency directories
foreach my $dir (split("\n", `find $options{p} -name .deps -a -type d`)) {
    $dir =~ s/\.deps$//;
    $directories{Cwd::realpath($dir)} = 1;
}


##############################################################################
# Create hash table where any dependency is remembered only once
# and store file modification time as hash value
foreach my $dir (keys %directories) {
    foreach my $file (split("\n", `ls $dir/.deps/`)) {
	open(DEP, "<$dir/.deps/$file") or
	    die("Cannot open $dir/.deps/$file for reading ($!).");
	while (<DEP>) {
	    chomp();

	    # remove dependency name, if any
	    s/^[^:]+://;

	    # remove trailing backslash
	    s/\s+\\$//;

	    foreach (split(" ", $_)) {

		# Remove starting and ending spaces
		s/^\s+//;
		s/\s+$//;

		# Only want to know about dependencies outside the build dir
		# Optionally, could check ony for:
		#	/usr/include/xorg
		#	/usr/include/X11
		#	pixman, dri, mesa, etc related headers
		# But at first, check everything, including libc and gcc
		# headers.
		if (/^\// and not /^$options{p}/) {
		    my $st = File::stat::stat($_);
		    $dependencies{$_} = $st->mtime;
		}
	    }
	}
	close(DEP);
    }
}


##############################################################################
# Check old dependency file if any specified
my $olddep = $options{'d'};
if ($olddep) {
    $olddep = Cwd::realpath($olddep);
    open(DEP, "<$olddep") or die("Cannot open $olddep for reading ($!).");
    while (<DEP>) {
	chomp;
	# Remove comments
	s/#.*//;	
	my ($name, $mtime) = split(':', $_);
	if ($name and $mtime) {
	    $olddeps{$name} = $mtime;
	    if ($mtime < $dependencies{$name}) {
		print("$name is newer.\n");
	    }
	}
    }
    close(DEP);
    # Check possible new dependencies
    foreach (keys %dependencies) {
	print("$_ is a new dependency.\n") unless $olddeps{$_};
    }
    unless ($options{'s'}) {
	# Check possible dependencies no longer required
	foreach (keys %olddeps) {
	    print("# $_ no longer required.\n") unless $dependencies{$_};
	}
    }
}


##############################################################################
# Write dependencies file, possibly printing warnings about files on
# non standard directories, or rpm package of dependency file
my $depfile = "$options{p}/$options{n}.deps";
if ($depfile eq $olddep) {
    # Better avoid risk of overwriting file,
    # besides there always exist a safe copy
    print(STDERR "Not replacing dependency file.\n");
    exit(1);
}


##############################################################################
sub query_file_package {
    my ($file) = @_;

    my $pkg = `rpm -qf $file`;
    chomp($pkg);
    $pkg =~ s/^\s+//;
    $pkg =~ s/\s+$//;

    push(@{$pkgdeps{$pkg}}, $file);

    return ($pkg);
};


##############################################################################
unless ($options{'c'}) {
    open(DEP, ">$depfile") or
	die("Cannot open $depfile file for writing ($!).");
    foreach (keys %dependencies) {
	if (not /^\/usr\/include/ and not /^\/usr\/lib\/gcc/) {
	    # Print comment about it in deps file
	    print(DEP "# Not standard $_\n");
	    # And also to stderr
	    print(STDERR "Warning: Not a standard dep $_\n") unless $options{'s'};
	}
	print(DEP "$_:$dependencies{$_}");
	## FIXME: Should not run rpm -qf for every file, as it returns
	## a list in the order of arguments, just need to parse it in
	## a proper way for possible problems with files not in any
	## package, and make sure it will not give rpm a command line
	## argument that may be too long and cause an error.
	print(DEP "\t# ", query_file_package($_)) unless $options{'f'};
	print(DEP "\n");
    }
    close(DEP);
}

unless ($options{'f'}) {
    my $pkgdepfile = "$options{p}/$options{n}-$package_system.deps";
    open(DEP, ">$pkgdepfile") or
	die("Cannot open $pkgdepfile file for writing ($!).");
    while (my ($dep, $list) = each(%pkgdeps)) {
	print(DEP $dep);
	my $first = 1;
	foreach (@$list) {
	    if ($first == 1) {
		print(DEP "\t# ");
		$first = 0;
	    }
	    else {
		print(DEP ", ");
	    }
	    print(DEP $_);
	}
	print(DEP "\n");
    }
    close(DEP);
}
